#!/usr/bin/env python


def mapattrs(output, *args):
    mapoutput = []
    for arg in args:
        for ele in output:
            if arg in ele:
                mapoutput.append(ele[arg])
    return list(set(mapoutput))


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'mapmultipleattr': mapattrs
        }



#!/usr/bin/env python

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from .InventoryFilter import InventoryFilter


class DummyInventory(InventoryFilter):
    def get_host_data(self, res, cfgs={}):
        """
        Returns a dict of hostnames or IP addresses for use in an Ansible
        inventory file, based on available data. Only a single hostname or IP
        address will be returned per instance, so as to avoid duplicate runs of
        Ansible on the same host via the generated inventory file.

        :param topo:
            linchpin Dummy/nummy resource data

        :param cfgs:
            map of config options from PinFile
        """
        host_data = {}
        var_data = cfgs.get('dummy', {})
        var_data.update(cfgs.get('nummy', {}))
        if var_data is None:
            var_data = {}
            var_data['__IP__'] = '__SELF__'
        for host in res['hosts']:
            host_data[host] = {}
            self.set_config_values(host_data[host], host, res, var_data)
        return host_data

    def get_host_ips(self, host_data):
        return host_data.keys()

    def get_inventory(self, topo, layout, config):
        host_data = self.get_host_data(topo, config)
        inven_hosts = self.get_host_ips(host_data)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO()
        self.config.write(output)
        return output.getvalue()

    def set_config_values(self, host_data, host, instance, cfgs={}):
        """
        """
        if cfgs is None:
            return
        if 'hostname' not in cfgs.keys():
            cfgs['hostname'] = '__IP__'
        for var in cfgs.keys():
            if var == 'hostname' and cfgs[var] == '__IP__':
                host_data[cfgs[var]] = host
            else:
                host_data[var] = self.config_value_helper(instance, cfgs[var])



import os
import yaml
import json

from cerberus import Validator

from linchpin.ansible_runner import ansible_runner
from linchpin.exceptions import HookError
from linchpin.hooks.action_managers.action_manager import ActionManager


class AnsibleActionManager(ActionManager):

    def __init__(self, name, action_data, target_data, **kwargs):

        """
        AnsibleActionManager constructor
        :param name: Name of Action Manager , ( ie., ansible)
        :param action_data: dictionary of action_block
        consists of set of actions
        example:
        - name: nameofthehook
          type: ansible
          actions:
            - playbook: test_playbook.yaml
              vars: test_var.yaml
              extra_vars: { "test_var": "postdowntask"}
        :param target_data: Target specific data defined in PinFile
        :param kwargs: anyother keyword args passed as metadata
        """

        self.name = name
        self.action_data = action_data
        self.target_data = target_data
        self.context = kwargs.get("context", True)
        self.kwargs = kwargs


    def validate(self):

        """
        Validates the action_block based on the cerberus schema
        example:: ansible_action_block::::
        - name: build_openshift_cluster
          type: ansible
          actions:
            - playbook: test_playbook.yaml
              vars: test_var.yaml
              extra_vars: { "testvar": "world"}
        """

        schema = {
            'name': {'type': 'string', 'required': True},
            'type': {'type': 'string', 'allowed': ['ansible']},
            'path': {'type': 'string', 'required': False},
            'context': {'type': 'boolean', 'required': False},
            'actions': {
                'type': 'list',
                'schema': {
                    'type': 'dict',
                    'schema': {
                        'playbook': {'type': 'string', 'required': True},
                        'vars': {'type': 'string', 'required': False},
                        'extra_vars': {'type': 'dict', 'required': False}
                    }
                },
                'required': True
            }
        }

        v = Validator(schema)
        status = v.validate(self.action_data)

        if not status:
            raise HookError("Invalid syntax: {0}".format((v.errors)))
        else:
            return status


    def load(self):

        """
        Loads the ansible specific managers and loaders
        """

    def get_ctx_params(self):

        """
        Reformats the ansible specific variables
        """

        ctx_params = {}
        ctx_params["resource_file"] = (
            self.target_data.get("resource_file", None))
        ctx_params["layout_file"] = self.target_data.get("layout_file", None)
        ctx_params["inventory_file"] = (
            self.target_data.get("inventory_file", None))

        return ctx_params


    def execute(self):

        """
        Executes the action_block in the PinFile
        """

        self.load()
        extra_vars = {}
        runners = []

        for action in self.action_data["actions"]:
            path = self.action_data["path"]
            playbook = action.get("playbook")

            if not(os.path.isfile(playbook)):
                playbook = "{0}/{1}".format(path, playbook)

            if "vars" in action:
                var_file = "{0}/{1}".format(path, action.get("vars"))
                ext = var_file.split(".")[-1]
                extra_vars = open(var_file, "r").read()

                if ("yaml" in ext) or ("yml" in ext):
                    extra_vars = yaml.load(extra_vars)
                else:
                    extra_vars = json.loads(extra_vars)

            e_vars = action.get("extra_vars", {})
            extra_vars.update(e_vars)
            verbosity = self.kwargs.get('verbosity', 1)

            if self.context:
                extra_vars.update(self.get_ctx_params())
            if 'inventory_file' in self.target_data and self.context:
                inv_file = self.target_data["inventory_file"]
                runners.append(ansible_runner(playbook,
                                              "",
                                              extra_vars,
                                              inventory_src=inv_file,
                                              verbosity=verbosity))
            else:
                runners.append(ansible_runner(playbook,
                                              "",
                                              extra_vars,
                                              inventory_src="localhost",
                                              verbosity=verbosity))
        return runners



#!/usr/bin/env python


def add_res_type(hosts, res_type):
    new_hosts = []
    for host in hosts:
        host['resource_type'] = res_type
        new_hosts.append(host)
    return new_hosts


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'add_res_type': add_res_type
        }



#!/usr/bin/env python


def prepare_ssh_args(ssh_args, users, sshkey):
    # "{{ ssh_args }} --ssh-inject \
    # {{ item.0}}:string:'{{ pubkey_local.stdout }}'"
    for user in users:
        ssh_args += "--ssh-inject " + user + ":string:'" + sshkey + "' "
    return ssh_args


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'prepare_ssh_args': prepare_ssh_args
        }
